@@ -11,7 +11,7 @@ use IO::Dir;
 use Path::Class ();
 use IO::File;
 
-our $VERSION = 0.06;
+our $VERSION = 0.07;
 
 =head1 NAME
 
@@ -78,7 +78,8 @@ C<dirs> or C<both>:
 
  $mdl->list(mode => 'both')
 
-To only get files/dirs directly under the current dir specify a C<recurse> option of 0.
+To only get files/dirs directly under the current dir specify a C<recurse>
+option of 0.
 
 Please note: the exact order in which files and directories are listed will
 change from OS to OS.
@@ -97,14 +98,14 @@ sub list {
     if ($opt{recurse}) {
         $self->{_dir}->recurse(callback => sub {
             my ($entry) = @_;
-            push @files, $entry->relative($self->{_dir})
+            push @files, $entry
                 if !$entry->is_dir && $opt{file} 
                 || $entry->is_dir && $opt{dir};
         });
-        return @files;
+        return map { $self->_rebless($_) } @files;
     }
 
-    @files = map {$_->relative($self->{_dir}) } $self->{_dir}->children;
+    @files = map { $self->_rebless($_) } $self->{_dir}->children;
 
     return @files if $opt{dir} && $opt{file};
 
@@ -114,6 +115,21 @@ sub list {
 
 }
 
+sub _rebless {
+  my ($self, $entity) = @_;
+
+  $entity = $entity->absolute($self->{root_dir});
+  if ($entity->is_dir) {
+    bless $entity, 'Catalyst::Model::File::Dir';
+  }
+  else {
+    bless $entity, 'Catalyst::Model::File::File';
+  }
+
+  $entity->{stringify_as} = $entity->relative($self->{_dir})->stringify;
+  return $entity;
+}
+
 =head2 change_dir
 
 =head2 cd
@@ -197,7 +213,9 @@ sub parent {
 
 =head2 $self->file($file)
 
-Returns an L<Class::Path::File> object of $file (which can be a string or a Class::Path::File object,) or undef if the file is an invalid path - i.e. outside the directory structure specified in the config.
+Returns an L<Path::Class::File> object of $file (which can be a string or a
+Class::Path::File object,) or undef if the file is an invalid path - i.e.
+outside the directory structure specified in the config.
 
 =cut
 
@@ -220,12 +238,12 @@ sub file {
 
 Shortcut to $self->file($file)->slurp.
 
-In a scalar context, returns the contents of $file in a string.  In a list context,
-returns the lines of $file (according to how $/ is set) as a list.  If the file can't be
-read, this method will throw an exception.
+In a scalar context, returns the contents of $file in a string.  In a list
+context, returns the lines of $file (according to how $/ is set) as a list.  If
+the file can't be read, this method will throw an exception.
 
-If you want "chomp()" run on each line of the file, pass a true value for the "chomp" or
-"chomped" parameters:
+If you want "chomp()" run on each line of the file, pass a true value for the
+"chomp" or "chomped" parameters:
 
  my @lines = $self->slurp($file, chomp => 1);
 
@@ -240,7 +258,8 @@ sub slurp {
 
 =head2 $self->splat($file, PRINT_ARGS)
 
-Does a print to C<$file> with the specified C<PRINT_ARGS>. Does the same as C<$self->file->openw->print(@_)>
+Does a print to C<$file> with the specified C<PRINT_ARGS>. Does the same as
+C<$self->file->openw->print(@_)>
 
 =cut
 
@@ -250,6 +269,52 @@ sub splat {
     $file->openw->print(@_);
 }
 
+package #
+   Catalyst::Model::File::File;
+use base 'Path::Class::File';
+sub stringify {
+  return $_[0]->{stringify_as} || $_[0]->abs_stringify;
+}
+
+sub abs_stringify {
+  Path::Class::File::stringify(shift)
+}
+
+# All these would probably be better done with Moose or something, but i'm lazy
+sub open {
+  my $s = shift;
+  local $s->{stringify_as};
+  return $s->SUPER::open(@_);
+}
+
+sub touch {
+  my $s = shift;
+  local $s->{stringify_as};
+  return $s->SUPER::touch(@_);
+}
+
+sub remove {
+  my $s = shift;
+  local $s->{stringify_as};
+  return $s->SUPER::touch(@_);
+}
+
+sub stat {
+  my $s = shift;
+  local $s->{stringify_as};
+  return $s->SUPER::stat(@_);
+}
+sub lstat {
+  my $s = shift;
+  local $s->{stringify_as};
+  return $s->SUPER::lstat(@_);
+}
+
+@Catalyst::Model::File::Dir::ISA = 'Path::Class::Dir';
+sub Catalyst::Model::File::Dir::stringify {
+  return $_[0]->{stringify_as}
+      || Path::Class::Dir::stringify($_[0]);
+}
 
 =head1 AUTHOR
 
